#!/usr/bin/python
#
# This software may be freely redistributed under the terms of the GNU
# library public license.
#
# You should have received a copy of the GNU Library Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import sys, os

if ((len(sys.argv) < 2) or (not os.path.isdir(sys.argv[1]))):
    sys.stderr.write('run kadischi.py with the wrapper script\n')
    sys.exit(1)

installdir = sys.argv[1]

if os.environ.has_key("KADISCHIARGS"):
    theargs = string.split(os.environ["KADISCHIARGS"])
else:
    theargs = sys.argv[2:]

# For anaconda in test mode
if (os.path.exists('isys')):
    # TODO fix test mode
    sys.path.append('isys')
    sys.path.append('textw')
    sys.path.append('iw')
else:
    sys.path.insert(0, '%s/lib/kadischi' % installdir)
    # NOTE this could also be a version of Anaconda that is distributed 
    # along with Kadischi
    sys.path.append('/usr/lib/anaconda')
    sys.path.append('/usr/lib/anaconda/textw')
    sys.path.append('/usr/lib/anaconda/iw')

if (os.path.exists('booty')):
    sys.path.append('booty')
    sys.path.append('booty/edd')
else:
    sys.path.append('/usr/lib/booty')

sys.path.append('/usr/share/system-config-keyboard')

import signal, traceback, string, isys, iutil

from exception import handleException
import dispatch
from flags import flags
from anaconda_log import anaconda_log

from rhpl.log import log
from rhpl.translate import _, textdomain

textdomain("anaconda")

# reset python's default SIGINT handler
signal.signal(signal.SIGINT, signal.SIG_DFL)

# we can't let the LD_PRELOAD hang around because it will leak into
# rpm %post and the like.  ick :/
if os.environ.has_key("LD_PRELOAD"):
    del os.environ["LD_PRELOAD"]

try:
    (args, extra) = isys.getopt(theargs, 'CGTRtdr:m:',
          [ 'graphical', 'text', 'test', 'debug', 'method=', 'rootpath=',
	    'traceonly', 'kickstart=', 'lang=', 'expert', 'autostep',
            'noselinux', 'selinux', 'cmdline', 'logfile='])
except TypeError, msg:
    sys.stderr.write("Error %s\n" % str(msg).strip())
    sys.exit(-1)

if extra:
    sys.stderr.write("Unexpected arguments: %s\n" % extra)
    sys.exit(-1)

# Save the arguments in case we need to reexec anaconda for kon
os.environ["ANACONDAARGS"] = string.join(sys.argv[1:])

# Parameters for the main anaconda routine
#
rootPath = None
logFile = None
debug = 0			# start up pdb immediately
traceOnly = 0			# don't run, just list modules we use
instClass = None		# the install class to use
method = None			# URL representation of install method
display_mode = None

#
# lang       - language to use for install/machine default
#
lang = None
method = None
ksfile = None

#
# parse off command line arguments
#
for n in args:
    (str, arg) = n

    if (str == '-d' or str == '--debug'):
	debug = 1
    elif (str == '--expert'):
	flags.expert = 1
    elif (str == '-G' or str == '--graphical'):
	display_mode = 'g'
    elif (str == '--kickstart'):
	from kickstart import Kickstart
	ksfile = arg
        instClass = Kickstart(ksfile, flags.serial)
    elif (str == '--lang'):
        lang = arg
    elif (str == '-m' or str == '--method'):
	method = arg
    elif (str == '--noselinux'):
        flags.selinux = 0
    elif (str == '--selinux'):
        flags.selinux = 1
    elif (str == "--autostep"):
	flags.autostep = 1
    elif (str == '-r' or str == '--rootpath'):
	rootPath = arg
    elif (str == '--logfile'):
	logFile = arg
    elif (str == '--traceonly'):
	traceOnly = 1
    elif (str == '-t' or str == '--test'):
	flags.test = 1
    elif (str == '-T' or str == '--text'):
        display_mode = 't'
    elif (str == "-C" or str == "--cmdline"):
        display_mode = 'c'

# set up anaconda logging
if not logFile:
    logFile = '/dev/null'
anaconda_log.open (logFile)
log.handler=anaconda_log

#
# must specify install, rescue mode
#

if (not method):
    sys.stderr.write('no install method specified\n')
    sys.exit(1)

if (not rootPath):
    sys.stderr.write('no rootpath directory specified\n')
    sys.exit(1)

if (os.path.exists(rootPath)):
    sys.stderr.write('rootpath directory already exists\n')
    sys.exit(1)

try:
    os.mkdir(rootPath, 0755)
except:
    sys.stderr.write('could not create rootpath directory\n')
    sys.exit(1)

if display_mode is None:
    sys.stderr.write('no display mode specified\n')
    sys.exit(1)

# flags for installing to a root path
flags.setupFilesystems = 0
flags.rootpath = 1

if (debug):
    import pdb
    pdb.set_trace()

# handle traceonly and exit
if traceOnly:
    if display_mode == 'g':
        sys.stderr.write("traceonly is only supported for text mode\n")
        sys.exit(0)

    # prints a list of all the modules imported
    from text import InstallInterface
    from text import stepToClasses
    import pdb
    import warnings
    import image
    import harddrive
    import urlinstall
    import mimetools
    import mimetypes
    import syslogd
    import installclass
    import re
    import rescue
    import kickstart
    import whiteout
    import findpackageset
    import libxml2
    import cmdline
    import encodings.utf_8
    import encodings.idna

    installclass.availableClasses()

    if display_mode == 't':
        for step in stepToClasses.keys():
            if stepToClasses[step]:
                (mod, klass) = stepToClasses[step]
                exec "import %s" % mod

    for module in sys.__dict__['modules'].keys ():
        if module not in [ "__builtin__", "__main__" ]:
            foo = repr (sys.__dict__['modules'][module])
            bar = string.split (foo, "'")
            if len (bar) > 3:
                print bar[3]

    sys.exit(0)

log("Display mode = %s", display_mode)
log("Method = %s", method)

#
# if no instClass declared by user figure it out based on other cmdline args
#
if not instClass:
    from installclass import DefaultInstall, availableClasses
    instClass = DefaultInstall(flags.expert)

    if len(availableClasses(showHidden = 1)) < 2:
        (cname, cobject, clogo) = availableClasses(showHidden = 1)[0]
        log("%s is only installclass, using it" %(cname,))
        instClass = cobject(flags.expert)

# this lets install classes force text mode instlls
if instClass.forceTextMode:
    dup_log(_("Install class forcing text mode installation"))
    display_mode = 't'

#
# display splash screen
#
if (display_mode == 'g'):
    print _("Starting graphical installation...")
    try:
        from splashscreen import splashScreenShow
        splashScreenShow()
        from gui import InstallInterface
    except Exception, e:
        dup_log("Exception starting GUI installer: %s" %(e,))
        sys.exit(1)

if (display_mode == 't'):
    from text import InstallInterface

if (display_mode == 'c'):
    from cmdline import InstallInterface

# go ahead and set up the interface
intf = InstallInterface ()

# imports after setting up the path
if method:
    if method.startswith('cdrom://'):
        from image import CdromInstallMethod
        methodobj = CdromInstallMethod(method[8:], rootPath, intf)
    elif method.startswith('nfs:/'):
        from image import NfsInstallMethod
        methodobj = NfsInstallMethod(method[5:], rootPath, intf)
    elif method.startswith('nfsiso:/'):
        from image import NfsIsoInstallMethod
        methodobj = NfsIsoInstallMethod(method[8:], rootPath, intf)
    elif method.startswith('ftp://') or method.startswith('http://'):
        from urlinstall import UrlInstallMethod
        methodobj = UrlInstallMethod(method, rootPath, intf)
    elif method.startswith('hd://'):
        from harddrive import HardDriveInstallMethod
        methodobj = HardDriveInstallMethod(method[5:], rootPath, intf)
    else:
        print "unknown install method:", method
        sys.exit(1)

id = instClass.installDataClass([], None, method)
id.x_already_set = 1

import rhpl.keyboard as keyboard
kbd = keyboard.Keyboard()
id.setKeyboard(kbd)

instClass.setInstallData(id)

dispatch = dispatch.Dispatcher(intf, id, methodobj, rootPath)

if lang:
    dispatch.skipStep("language", permanent = 1)
    instClass.setLanguage(id, lang)
    instClass.setLanguageDefault(id, lang)

# Skip the disk options in rootpath mode
if flags.rootpath:
    dispatch.skipStep("welcome", permanent = 1)
    dispatch.skipStep("partitionmethod", permanent = 1)
    dispatch.skipStep("partitionmethodsetup", permanent = 1)
    dispatch.skipStep("autopartition", permanent = 1)
    dispatch.skipStep("autopartitionexecute", permanent = 1)
    dispatch.skipStep("fdisk", permanent = 1)
    dispatch.skipStep("bootloadersetup", permanent = 1)
    dispatch.skipStep("bootloader", permanent = 1)
    dispatch.skipStep("bootloaderadvanced", permanent = 1)
    dispatch.skipStep("upgbootloader", permanent = 1)
    dispatch.skipStep("partitionobjinit", permanent = 1)
    dispatch.skipStep("partition", permanent = 1)
    dispatch.skipStep("addswap", permanent = 1)
    dispatch.skipStep("partitiondone" , permanent = 1)
    dispatch.skipStep("instbootloader", permanent = 1)
    dispatch.skipStep("bootdisk", permanent = 1)
    dispatch.skipStep("makebootdisk" , permanent = 1)
    dispatch.skipStep("writexconfig" , permanent = 1)
    dispatch.skipStep("writeksconfig" , permanent = 1)
    dispatch.skipStep("complete" , permanent = 1)

instClass.setSteps(dispatch)

# don't install locales
# TODO make this configurable
import rpm
rpm.addMacro("_install_langs", "%{nil}")

# comment out the next line to make exceptions non-fatal
sys.excepthook = lambda type, value, tb, dispatch=dispatch, intf=intf: handleException(dispatch, intf, (type, value, tb))

try:
    intf.run(id, dispatch)
    pass
except SystemExit, code:
    intf.shutdown()
except:
    handleException(dispatch, intf, sys.exc_info())

del intf

try:
    isys.umount(rootPath + '/proc', removeDir = 0)
except:
    pass

# add USB boot device to /etc/fstab
import fsset
ff = fsset.FileSystemSet()
fsystem = fsset.fileSystemTypeGet("ext3")
device = fsset.DevDevice("")
entry = fsset.FileSystemSetEntry(device, "/", fsystem, "defaults")
entry.setLabel("FCUSB")
entry.fsck = 0
ff.add(entry)
f = open (rootPath + "/etc/fstab", "w")
f.write (ff.fstab())
f.close ()

# TODO install the readonly-root stuff
# TODO create initrd containing USB modules
# TODO run prelink in chroot
# TODO run depmod -a in chroot for each kernel
# TODO anything else?
